import React, { Component, ReactElement, ReactNode } from 'react';

// k/v Obj
export interface Obj {
  [key: string]: unknown;
}

export interface ICnColumnStaticPart {
  /** 列的名称 */
  name: string;

  /** 在数据中的字段 code */
  code?: string;
  dataIndex?: string;
  key?: string;

  /** 列标题的展示名称；在页面中进行展示时，该字段将覆盖 name 字段 */
  title?: ReactNode;

  /** 列的宽度，如果该列是锁定的，则宽度为必传项 */
  width?: number;

  /** 是否锁列 */
  lock?: boolean;

  // 是否隐藏
  hidden?: boolean;

  /** 功能开关 */
  features?: { [key: string]: any };

  symbol?: string | symbol;

  // false 则关闭拖拽
  resizable?: boolean;

  // 最大宽度，undefined 会走全局默认最大宽度460， false则关闭最大宽度， number则是独立的最大宽度
  maxWidth?: number | false;

  /**
   * 是否支持自适应，默认是true，可以手动关闭
   */
  enableAutoWidth?: boolean;

  attach?:
    | boolean
    | {
        // 该列在列随操作中是否可被复制
        copyEnable?: boolean;
        copyTransform?: () => string;
      };
}

export interface ICnColumnDynamicPart {
  /** 自定义取数方法 */
  getValue?(row: any, rowIndex: number): any;

  /** 自定义渲染方法 */
  render?(value: any, row: any, rowIndex: number): ReactNode;
  cell?(value: any, rowIndex: number, row: any): ReactNode;
}

export interface ICnColumn extends ICnColumnStaticPart, ICnColumnDynamicPart {
  /** 该列的子节点 */
  children?: ICnColumn[];
  sizeFixed?: boolean; // 宽度是否固定
}
